"""
A reinforcement learning training script using FLOW (https://github.com/flow-project/flow). 

Model: PPO2
Agent: Single agent
Policy: MlpPolicy
Libraries (used by flow): stable-baselines, tensorflow, SUMO

Network:
2 x 2 traffic grid. 
Each edge has two lanes. Vehicles can turn and routes are random. 

"""

import warnings
from multiprocessing import Process, freeze_support
from helper import train, run, run_safe, setup_flow
from helper_old import run_old, setup_flow_old, train_old 
import argparse
from datetime import datetime
import random

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("mode", choices =["train", "run", "safe", "run-old", "train-old"] ,help="choose from [train, run, safe, run-old, train-old]", type=str)
    parser.add_argument("file", nargs="?", help="model file", type=str)
    parser.add_argument("--eval", nargs="?", help="number of random runs to evaluate a model", type=int, default=1)
    parser.add_argument("--render", nargs="?", choices=["true", "false"], help="whether to render SUMO gui", type=str, default="false")
    parser.add_argument("--rate", nargs="?", help="number of vehicles per hour", type=int, default=500)
    parser.add_argument("--yellow", nargs="?", help="specify the duration of yellow lights in safe controller", type=float, default=3)
    parser.add_argument("--straight", nargs="?", help="specify the duration of straight red/green lights in safe controller", type=float, default=26)
    parser.add_argument("--left", nargs="?", help="specify the duration of left-turning red/green lights in safe controller", type=float, default=5)
    parser.add_argument("--scenario", nargs="?", help="scenarios that trip the AI system", type=int, default=None)
    parser.add_argument("--route", nargs="?", help="routes that trip the AI system", type=int, default=None)
    parser.add_argument("--num-observed", nargs="?", help="number of vehicles observed on each edge (must be even)", type=int, default=6)
    parser.add_argument("--lr", nargs="?", help="set learning rate to <lr> * e-4", type=float, default=5e-4) 
    parser.add_argument("--model_path", nargs="?", help="path from train_systems_double_lanes to save files", type=str, default="")
    args = parser.parse_args()

    RENDER = False
    if args.render == "true":
        RENDER = True

    starting_step = 0
    model_file = args.file
    if model_file is not None: 
        print(model_file)
        print(model_file.strip("rl_model.zip"))
        starting_step = int(model_file.strip("rl_model.zip"))

    durations = {"yellow": args.yellow, "straight": args.straight, "left": args.left}

    if args.mode == "train":
        flow_params = setup_flow(args.rate, args.scenario, durations, args.num_observed, args.route, render=RENDER)
        print("starting from step", starting_step)
        train(flow_params, starting_step, model_file, args.lr, args.model_path)
    elif args.mode == "run":
        if model_file is not None: 
            flow_params = setup_flow(args.rate, args.scenario, durations, args.num_observed, args.route, render=RENDER)
            print("running from step", starting_step)
            run(flow_params, starting_step, model_file, args.eval)
        else:
            print("need to specify which model to run")
            exit(1)
    elif args.mode == "train-old":
        flow_params = setup_flow_old(args.rate, args.scenario, args.num_observed, args.route, render=RENDER)
        print("starting from step", starting_step)
        train_old(flow_params, starting_step, model_file)
    elif args.mode == "run-old":
        if model_file is not None: 
            flow_params = setup_flow_old(args.rate, args.scenario, args.num_observed, args.route, render=RENDER)
            print("running from step", starting_step)
            run_old(flow_params, starting_step, model_file, args.eval)
        else:
            print("need to specify which model to run")
            exit(1)
    elif args.mode == "safe":
        flow_params = setup_flow(args.rate, args.scenario, durations, args.num_observed, args.route, static=True, render=RENDER)
        run_safe(flow_params, args.eval)

if __name__ == '__main__':
    warnings.filterwarnings('ignore', category=FutureWarning)
    freeze_support()
    main()

# when training, generate a random seed. 
# in run, here's the seed to run recreate